/*									tab:4
 *
 * bitonic.h - Function prototypes and definitions for bitonic.sc
 *
 * "Copyright (c) 1993 and The Regents of the University 
 * of California.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Author: 			Andrea C. Dusseau
 * Version:			19
 * Creation Date:	Mon Aug  2 12:16:17 1993
 * Filename:		bitonic.h
 * History:
 */

#ifndef __BITONIC_H__
#define __BITONIC_H__

typedef enum {COMP_MIN, COMP_MAX} comp_t;
typedef enum {UP_POINT, DOWN_POINT, UP_MID, DOWN_MID, UNKNOWN_DIR} dir_point_t;
typedef enum {SINGLE_DOWN, SINGLE_UP, DOUBLE_DOWN, DOUBLE_UP, SLOPE_DOWN, 
		SLOPE_UP, IDENTICAL} bitonic_t;

void bitonic_sort(int np);
void print_results(int np);
void bitonic_local_radix_sort(int np, int *src, int *dest);


/* -------------------------------------------------------------------- 
 *
 * Routines to find the minimum of a bitonic sequence
 *
 * -------------------------------------------------------------------- 
 */
int find_bmin_double_up(int f, int l, int *s);
int find_bmin_double_down(int f, int l, int *s);
int inline find_bmin(int np, int *srcBuf);
int inline which_bitonic(int np, int *s);
int inline which_dir_point(int f, int f_dir, int l, int l_dir, int m, int *s);
int find_bmin_single_up(int first, int last, int *s);
int find_bmin_single_down(int first, int last, int *s);

/* -------------------------------------------------------------------- 
 *
 * Routines to sort a bitonic sequence.  With and witout gathering.
 *
 * -------------------------------------------------------------------- 
 */
void bmerge_down(int np, int *srcBuf, int *destBuf);
void bmerge_up(int np, int *srcBuf, int *destBuf);
void one_merge(int outerdim, int np, int *src, int *dest);
void bmerge_down_gather(int np, int *srcBuf, int *destBuf);
void bmerge_up_gather(int np, int *srcBuf, int *destBuf);
void one_merge_gather(int outerdim, int np, int *src, int *dest);
comp_t inline getMode(unsigned iproc, unsigned innerdim, unsigned outerdim);

/* -------------------------------------------------------------------- 
 *
 * Routines to perform butterly merge stages with cyclic mapping
 *
 * -------------------------------------------------------------------- 
 */

#define HALF_CACHE_BYTES (32*1024)

void cyclic_merge(unsigned outerdim, int np, int *s, int offset);
void cyclic_merge_cache(int np, int *s, int offset, int groups, int outerdim, 
			int innerdim);		


#endif /* __BITONIC_H__ */
