/*
 * sort.sc - Miscellaneous routines common to the sorting applications
 */
#include <stdio.h>
#include <split-c/split-c.h>
#include "sort.h"

/*
 * Procedure: create_keys
 * Parameters:
 *   np:      (input) number of keys in list
 *   col:     (output) a list of np keys with the given distribution
 *   distrib: (input) designates which input distribution (entropy) 
 *            should be used
 */
void create_keys(int np, int *col, int distrib)
{
  int i;

  srandom(MYPROC*3 + 17);
 
  switch (distrib) {
  case 0:                       /* Entropy = 31 */
    for (i = 0; i < np; i++)    
      col[i] = random(); 
    break;
  case 1:                       /* Entropy = 25.1 */
    for (i = 0; i < np; i++)    
      col[i] = random() & random();
    break;
  case 2:                       /* Entropy = 16.9 */
    for (i = 0; i < np; i++)    
      col[i] = random() & random() & random();
    break;
  case 3:                       /* Entropy = 10.4 */
    for (i = 0; i < np; i++)    
      col[i] = random() & random() & random() & random();
    break;
  case 4:                       /* Entropy = 6.2 */
    for (i = 0; i < np; i++)    
      col[i] = random() & random() & random() & random() & random();
    break;
  case 5:                       /* Entropy = 0 */
    for (i = 0; i < np; i++)    
      col[i] = 0;
    break;
  }
}

static check[PROCS]::;

/*
 * Procedure: all_check_results
 * Parameters: 
 *   np (input): number of keys per processor
 *   b (input): the list of allegedly sorted keys
 *   down (input): 0 -- the list should be increasing
 *                 1 -- the list is decreasing (bitonic sort)
 */
void all_check_results(int np, int *b, int down)
{
  int ii,jj;
  int error = 0;

  on_one printf("Checking correctness of sort. . .\n");
  /*
   * Check that local keys are sorted
   */
  if (!down) {
    for(ii = 0; ii < (np-1); ii++){
      if (b[ii] > b[ii+1]) {
        printf("Error at proc %d item %d\n",MYPROC,ii);
        error++;
      }
    }
  }
  else {
    for(ii = 0; ii < (np-1); ii++){
      if (b[ii] < b[ii+1]) {
        printf("Error at proc %d item %d\n",MYPROC,ii);
        error++;
      }
    }
  }

  /*
   * Check that the last key of processor i is less than or equal to
   * the first key of processor i+1.
   */
  if (!down) {
    check[MYPROC] = b[0];
    barrier();
    if (MYPROC < (PROCS - 1)) {
      if (b[np-1] > check[MYPROC+1]) {
        printf("proc %d: error across to  %d \n",MYPROC,MYPROC+1);  
        error++;
      }
    }
  }
  else {
    check[MYPROC] = b[0];
    barrier();
    if (MYPROC < (PROCS - 1)) {
      if (b[np-1] < check[MYPROC+1]) {
        printf("proc %d: error across to  %d \n",MYPROC,MYPROC+1);  
        error++;
      }
    }
  }
  error = all_reduce_to_one_add(error);
  on_one if (!error) printf(". . .keys sorted correctly.\n");
}
